# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

from .resources.transaction import Transaction, TransactionResponse


class Transactions(object):
    def __init__(self, client):
        self.requestor = client.requestor
        self.configuration = client.configuration

    def all(self, page=None, limit=None, before=None, after=None):
        filters = {}

        if page:
            filters['page'] = page

        if limit:
            filters['limit'] = limit

        if before:
            filters['created_before'] = before.strftime('%Y-%m-%d %H:%M:%S')

        if after:
            filters['created_after'] = after.strftime('%Y-%m-%d %H:%M:%S')

        status_code, response = self.requestor.request("get", "/api/v1/transactions/", query=filters)
        return TransactionResponse(
            list(map(Transaction, response.get('transactions', []))),
            response.get('metadata', {})
        )

    def detail(self, uuid):
        status_code, response = self.requestor.request("get", "/api/v1/transactions/%s" % uuid)
        return Transaction(response)

    def update(self, uuid, metadata):
        raise NotImplemented("Transaction update is not implemented")



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

import json
import hmac
from urllib.parse import urlencode

import requests

from hashlib import sha1
from .errors import APIConnectionError, APIError
from .utils import JSONEncoder


class Requestor:
    def __init__(self, client):
        self.client = client
        self.configuration = self.client.configuration

    def request(self, method, endpoint_url, raw=None, query=None):
        if not raw:
            raw = {}

        data = json.dumps(raw, cls=JSONEncoder)
        headers = {
            "User-Agent": self.client.identifier(),
            "Content-Type": "application/json",
            "X-Key": self.configuration.key,
        }

        if query:
            headers["X-Signature"] = self.sign("%s;%s" % (endpoint_url + "?" + urlencode(query), data))
            print(endpoint_url + "?" + urlencode(query))
        else:
            headers["X-Signature"] = self.sign("%s;%s" % (endpoint_url, data))

        print(self.configuration.environment.api_base_url + endpoint_url)

        try:
            response = requests.request(
                method=method,
                url=self.configuration.environment.api_base_url + endpoint_url,
                headers=headers,
                data=data,
                timeout=self.configuration.timeout,
                params=query
            )
        except Exception as e:
            self._handle_request_error(e)

        if not (200 <= response.status_code < 300):
            try:
                data = response.json()
            except ValueError as e:
                raise APIError(
                    "Invalid JSON response from API: %s (%s)"
                    % (str(e), response.text),
                    response.text,
                    response.status_code,
                )

            try:
                error_message = data["error"]
            except (KeyError, TypeError):
                raise APIError(
                    "Invalid response object from API: %s (HTTP response code "
                    "was %d)" % (response.text, response.status_code),
                    response.text,
                    response.status_code,
                )

            raise APIError(
                "%s (%s)" % (error_message, data["developer_message"]),
                response.text,
                response.status_code,
            )

        try:
            data = response.json()
        except ValueError as e:
            raise APIError(
                "Invalid JSON response from API: %s (%s)"
                % (str(e), response.text),
                response.text,
                response.status_code,
            )

        return response.status_code, data

    def client_ip(self, environ):
        try:
            return environ["HTTP_X_FORWARDED_FOR"].split(",")[0].strip()
        except (KeyError, IndexError):
            pass

        try:
            return environ["HTTP_X_REAL_IP"]
        except KeyError:
            pass

        return environ.get("REMOTE_ADDR")

    def sign(self, data):
        return hmac.new(
            self.configuration.secret.encode("utf-8"),
            data.encode("utf-8"),
            sha1,
        ).hexdigest()

    def _handle_request_error(self, e):
        if isinstance(e, requests.exceptions.RequestException):
            err = "%s: %s" % (type(e).__name__, str(e))
        else:
            err = "A %s was raised" % (type(e).__name__,)

            if str(e):
                err += " with error message %s" % (str(e),)
            else:
                err += " with no error message"

        msg = "Network error: %s" % (err,)

        raise APIConnectionError(msg)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, division, print_function

from .configuration import Configuration
from .errors import ConfigurationError
from .messages import Messages
from .payments import Payments
from .transactions import Transactions
from .requestor import Requestor
from .version import VERSION


class Cheddar(object):
    def __init__(self, configuration=None, **kwargs):
        if isinstance(configuration, Configuration):
            self.configuration = configuration
        else:
            raise ConfigurationError("Missing configuration object.")

        self.version = VERSION
        self.requestor = Requestor(self)
        self.payments = Payments(self)
        self.messages = Messages(self)
        self.transactions = Transactions(self)

    def identifier(self):
        return "cheddar-python/%s" % self.version



# -*- coding: utf-8 -*-

import os
import sys
from codecs import open
from setuptools import find_packages, setup
from setuptools.command.test import test as TestCommand


class PyTest(TestCommand):
    def initialize_options(self):
        TestCommand.initialize_options(self)

    def run_tests(self):
        import pytest

        errno = pytest.main([])
        sys.exit(errno)


current_path = os.path.abspath(os.path.dirname(__file__))
os.chdir(os.path.abspath(current_path))


version = {}
with open(
    os.path.join(current_path, "cheddar", "version.py"), encoding="utf-8"
) as f:
    exec(f.read(), version)

setup(
    name="cheddarpayments",
    version=version["VERSION"],
    description="Cheddar Python Bindings",
    long_description=open("./README.md", "r").read(),
    long_description_content_type="text/markdown",
    author="Palo Sopko",
    author_email="pavol.sopko@backbone.sk",
    url="https://www.backbone.sk/en/",
    license="MIT",
    keywords="cheddar, api, payments, money, cardpay, comfortpay, tatrapay, trustpay, sporopay, eplatby, gpwebpay, iterminal",
    packages=find_packages(exclude=["tests", "tests.*"]),
    install_requires=["requests >= 2.21.0", "six >= 1.3.0"],
    tests_require=["pytest >= 4.6.2, < 4.7"],
    cmdclass={"test": PyTest},
    project_urls={
        "Bug Tracker": "https://github.com/backbonesk/cheddar-python/issues",
        "Source Code": "https://github.com/backbonesk/cheddar-python",
    },
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
